package exion.game.runner;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.io.*;
import java.net.*;

import exion.game.runner.gui.RCanvas;

/**
 * Title:        RGame
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      exion
 * @author Matey Nenov
 * @version 1.0
 */

public class ERunner extends Applet implements Runnable, KeyListener
{
  public final static int NO_ACTION  = 0;
  public final static int MOVE_LEFT  = 1;
  public final static int MOVE_RIGHT = 1 << 1;
  public final static int MOVE_UP    = 1 << 2;
  public final static int MOVE_DOWN  = 1 << 3;

  public final static int IN_HOLE_TIME = 10;

  private Thread  thread       = null;
  private static boolean isStandalone = false;
  private boolean running      = false;
  private boolean reset        = false;
  private Table   tableL1      = null;
  private Table   tableL2      = null;
  private Table   tableL3      = null;
  private RCanvas canvas       = null;
  private Point   cp           = new Point(0, 0);
  private Point   start        = null;
  private boolean inHole       = false;
  private boolean canFallInHole = true;

  private Hashtable loadedLevels = new Hashtable();

  private boolean pause        = true;

  private int     speed        = 100;

  private int     inHoleTime   = 0;
  private int     mooving      = 0;

  private int     goldLeft     = 0;

  private int     currentLevel = 1;

  private Vector  enemies      = new Vector();
  private Vector  holes        = new Vector();

  private boolean first        = true;

  private Image [] images      = null;
  private String [] imageNames = {"background.jpg", "brick.jpg", "earth.jpg", "ledder.gif", "gate.gif", "diamond.gif", "hero.gif", "enemy.gif", "hole.gif"};

  /**Get a parameter value*/
  public String getParameter(String key, String def) {
    return isStandalone ? System.getProperty(key, def) :
      (getParameter(key) != null ? getParameter(key) : def);
  }

  /**Construct the applet*/
  public ERunner()
  {
    super();
    setLayout(new BorderLayout());
    addKeyListener(this);
  }

  /**Initialize the applet*/
  public void init()
  {
    images = new Image[imageNames.length];

    for(int i=0; i < imageNames.length; i++)
    {
      if(isStandalone)
        images[i] = Toolkit.getDefaultToolkit().getImage(imageNames[i]);
      else
        images[i] = getImage(getCodeBase(),  imageNames[i]);
    }

    initImages();

    jbInit();
  }

  public void initNextLevel()
  {
    if(loadedLevels.get("erlevel" + currentLevel + ".sav1") == null)
    {
      loadTable("erlevel" + currentLevel + ".sav");
      loadedLevels.put("erlevel" + currentLevel + ".sav1", tableL1.clone());
      loadedLevels.put("erlevel" + currentLevel + ".sav2", tableL2.clone());
      loadedLevels.put("erlevel" + currentLevel + ".sav3", tableL3.clone());
    }else
    {
      tableL1 = (Table)((Table)loadedLevels.get("erlevel" + currentLevel + ".sav1")).clone();
      tableL2 = (Table)((Table)loadedLevels.get("erlevel" + currentLevel + ".sav2")).clone();
      tableL3 = (Table)((Table)loadedLevels.get("erlevel" + currentLevel + ".sav3")).clone();
    }
    enemies.removeAllElements();
    holes.removeAllElements();
    goldLeft = 0;
    for(int i=0; i<tableL2.getWidth(); i++)
    {
      for(int j=0; j<tableL2.getHeight(); j++)
      {
        if(tableL3.getIcon(i, j) == Table.HERO1)
        {
          cp.move(i, j);
          start = new Point(cp.x, cp.y);
        }
        if(tableL2.getIcon(i, j) == Table.GOLD1)
          goldLeft++;
        if(tableL3.getIcon(i, j) == Table.ENEMY1)
        {
          System.out.println("Enemy found at: [" + i + ":" + j + "]");
          enemies.addElement(new Enemy(i, j, tableL1, tableL2, tableL3));
        }
      }
    }

    System.out.println("Gold to collect: " + goldLeft);

    if(canvas == null)
    {
      canvas = new RCanvas(tableL1, tableL2, tableL3);
      canvas.addKeyListener(this);

      canvas.setImages(images);
      canvas.setShowWires(false);
    }else
      canvas.initTables(tableL1, tableL2, tableL3);
      canvas.setDrawExit(false);
  }

  /**Invoked when a key has been pressed*/
  public void keyPressed(KeyEvent e)
  {
    pause = false;
    if(e.getKeyCode() == KeyEvent.VK_F)
    {
      speed -= (speed > 20)?20:0;
    }else if(e.getKeyCode() == KeyEvent.VK_S)
    {
      speed += (speed < 200)?20:0;
    }else if(e.getKeyCode() == KeyEvent.VK_END)
    {
      if(tableL1.getIcon(cp.x-1, cp.y+1) == Table.EARTH)
      {
        if(tableL2.getIcon(cp.x-1, cp.y+1) == Table.BACKGROUND)
        {
          holes.addElement(new Hole(cp.x-1, cp.y+1, holes, tableL2));
        }
      }
    }else if(e.getKeyCode() == KeyEvent.VK_PAGE_DOWN)
    {
      if(tableL1.getIcon(cp.x+1, cp.y+1) == Table.EARTH)
      {
        if(tableL2.getIcon(cp.x+1, cp.y+1) == Table.BACKGROUND)
        {
          holes.addElement(new Hole(cp.x+1, cp.y+1, holes, tableL2));
        }
      }
    }else
    {
      switch(e.getKeyCode())
      {
        case KeyEvent.VK_LEFT  : mooving |= ERunner.MOVE_LEFT; break;
        case KeyEvent.VK_RIGHT : mooving |= ERunner.MOVE_RIGHT; break;
        case KeyEvent.VK_UP    : mooving |= ERunner.MOVE_UP; break;
        case KeyEvent.VK_DOWN  : mooving |= ERunner.MOVE_DOWN; break;
      }
    }
  }

  /**Invoked when a key has been released*/
  public void keyReleased(KeyEvent e)
  {
    switch(e.getKeyCode())
    {
      case KeyEvent.VK_LEFT  : mooving &= ~ERunner.MOVE_LEFT; break;
      case KeyEvent.VK_RIGHT : mooving &= ~ERunner.MOVE_RIGHT; break;
      case KeyEvent.VK_UP    : mooving &= ~ERunner.MOVE_UP; break;
      case KeyEvent.VK_DOWN  : mooving &= ~ERunner.MOVE_DOWN; break;
    }
  }

  /**Invoked when a key has been typed*/
  public void keyTyped(KeyEvent e) {}

  /**Load current table from file*/
  private synchronized boolean loadTable(String fileName)
  {
    try{
      ObjectInputStream ois = null;

      if(isStandalone)
        ois = new ObjectInputStream(new FileInputStream(fileName));
      else
        ois = new ObjectInputStream(new URL(getCodeBase(), fileName).openStream());
      tableL1 = (Table)ois.readObject();
      tableL2 = (Table)ois.readObject();
      tableL3 = (Table)ois.readObject();
      ois.close();

      return true;
    }catch(IOException ioe){System.err.println(ioe.getMessage());}
    catch(ClassNotFoundException cnfe){System.err.println(cnfe.getMessage());}

    return false;
  }

  private void initImages()
  {
    MediaTracker mt = new MediaTracker(this);
    for(int i=0; i<images.length; i++)
      mt.addImage(images[i], i);
    System.out.println("Start loading all images");
    try{
      mt.waitForAll();
    }catch(InterruptedException ie){System.err.println(ie.getMessage());}
    System.out.println("All images loaded...");
  }

  /**Component initialization*/
  private void jbInit()
  {
/*    if(canvas != null)
      remove(canvas);
*/
    pause = true;

    initNextLevel();
    if(first)
    {
      first = false;

      add("Center", canvas);
      canvas.setVisible(true);
      canvas.initImages();
    }
  }

  /**Draw to screen*/
  public void paint(Graphics g)
  {
  }

  /**Main thread*/
  public synchronized void run()
  {
    int i = 0;
    do{
      reset = false;

      while(!canvas.isInitialized())
      {
        canvas.initImages();
        try{
          thread.sleep(100);
        }catch(InterruptedException ie){System.out.println(ie.getMessage());}
      }

      canvas.initBackgroundImage();

      while(running)
      {
        computeMove();
        if(!reset)
        {
          canvas.initCurrentFrame();
          canvas.updateFrame();
          canvas.repaintBackBuffer();
          do{
            try{
              thread.sleep(speed);
            }catch(InterruptedException ie){System.out.println(ie.getMessage());}
            if(i++ >= 1)
            {
              canvas.initCurrentFrame();
              canvas.updateFrame();
              canvas.repaintBackBuffer();

              if(canvas.getParent() == null)
              this.add("Center", canvas);
              doLayout();
            }
          }while(pause);
          i=0;
        }
      }
      running = true;
      doLayout();
    }while(reset);
  }

  /**Start the applet*/
  public void start()
  {
    running = true;
    reset = true;
    thread = new Thread(this);
    thread.start();
  }
  /**Stop the applet*/
  public void stop()
  {
    running = false;
    reset = false;
  }
  /**Destroy the applet*/
  public void destroy()
  {
  }

  /**Get Applet information*/
  public String getAppletInfo() {
    return "Applet Information";
  }
  /**Get parameter info*/
  public String[][] getParameterInfo() {
    return null;
  }
  /**Main method*/
  public static void main(String[] args)
  {
    ERunner.isStandalone = true;
    ERunner applet = new ERunner();
    Frame frame;
    frame = new Frame() {
      protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
          System.exit(0);
        }
      }
      public synchronized void setTitle(String title) {
        super.setTitle(title);
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
      }
    };
    frame.setTitle("Applet Frame");
    frame.add(applet, BorderLayout.CENTER);
    applet.init();
    applet.start();
    frame.pack();
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setLocation((d.width - frame.getSize().width) / 2, (d.height - frame.getSize().height) / 2);
    frame.setVisible(true);
  }

  /**Compute next move*/
  private synchronized void computeMove()
  {
    boolean mooved = false;

    for(int i=0; i<holes.size(); i++)
      ((Hole)holes.elementAt(i)).update();
    for(int i=0; i<enemies.size(); i++)
      ((Enemy)enemies.elementAt(i)).attack(holes);

    for(int i=0; i<enemies.size(); i++)
    {
      if(((Enemy)enemies.elementAt(i)).x == cp.x && ((Enemy)enemies.elementAt(i)).y == cp.y)
      {
        System.out.println("You Lose!!!");
        stop();
        reset = true;
        jbInit();

        return;
      }
    }

    if(cp.y < tableL1.getHeight() - 2)
    {
      if(tableL1.getIcon(cp.x, cp.y+1) == Table.BACKGROUND || (((mooving & ERunner.MOVE_DOWN)  != 0) && (tableL1.getIcon(cp.x, cp.y+1) == Table.LEDDERS)))
      {
        tableL3.setIcon(cp.x, cp.y++, 0);

        if(inHole)
        {
          inHole = false;
          inHoleTime = 0;
        }

        mooved = true;
      } else if(tableL2.getIcon(cp.x, cp.y+1) == Table.HOLE1 && tableL3.getIcon(cp.x, cp.y+1) == Table.BACKGROUND && canFallInHole)
      {
        inHole = true;
        inHoleTime = IN_HOLE_TIME;
        tableL3.setIcon(cp.x, cp.y++, Table.BACKGROUND);
        canFallInHole = false;

        mooved = true;
      }
    }

    if(inHole)
    {
      if(tableL2.getIcon(cp.x, cp.y) != Table.HOLE1)
      {
        inHoleTime = 0;
        inHole = false;
        tableL3.setIcon(cp.x, cp.y, Table.BACKGROUND);
        cp.move(start.x, start.y);

      } else
      {
        if(--inHoleTime == 0)
        {
          tableL3.setIcon(cp.x, cp.y--, Table.BACKGROUND);
          inHoleTime = 0;
          inHole = false;
        }
      }
      mooved = true;
    }

    if(!mooved)
    {
      if((mooving & ERunner.MOVE_UP) != 0)
      {
        canFallInHole = true;
        if(cp.y > 0)
          if(tableL1.getIcon(cp.x, cp.y-1) == Table.LEDDERS || ((tableL1.getIcon(cp.x, cp.y) == Table.LEDDERS) && (tableL1.getIcon(cp.x, cp.y-1) == Table.BACKGROUND)))
            tableL3.setIcon(cp.x, cp.y--, 0);
      }
      if((mooving & ERunner.MOVE_LEFT) != 0)
      {
        canFallInHole = true;
        if(cp.x > 0)
          if(tableL1.getIcon(cp.x-1, cp.y) == Table.BACKGROUND || tableL1.getIcon(cp.x-1, cp.y) == Table.LEDDERS)
            tableL3.setIcon(cp.x--, cp.y, 0);
      }
      if((mooving & ERunner.MOVE_RIGHT) != 0)
      {
        canFallInHole = true;
        if(cp.x < tableL1.getWidth() - 2)
          if(tableL1.getIcon(cp.x+1, cp.y) == Table.BACKGROUND || tableL1.getIcon(cp.x+1, cp.y) == Table.LEDDERS)
            tableL3.setIcon(cp.x++, cp.y, 0);
      }
    }

    if(tableL2.getIcon(cp.x, cp.y) == Table.EXIT1)
    {
      if(goldLeft == 0)
      {
        currentLevel++;
//        stop();
//        reset = true;
        jbInit();

        return;
      }
    }

    if(tableL2.getIcon(cp.x, cp.y) == Table.GOLD1)
    {
      tableL2.setIcon(cp.x, cp.y, Table.BACKGROUND);
      System.out.println("Gold collected!!!");
      System.out.println("Gold left: " + --goldLeft);
      if(goldLeft == 0)
      {
        canvas.setDrawExit(true);
      }
    }

    tableL3.setIcon(cp.x, cp.y, Table.HERO1);
  }
}