package exion.game.runner.gui;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Point;

import exion.game.runner.Table;

/**
 * Title:        RGame
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      exion
 * @author Matey Nenov
 * @version 1.0
 */

public class RCanvas extends Canvas
{
  private Image [] images             = null;
  private Image    background         = null;
  private Graphics backgroundGraphics = null;
  private Image    cFrame             = null;
  private Graphics cFrameGraphics     = null;
  private Graphics thisGraphics       = null;
  private Table    tableL1            = null;
  private Table    tableL2            = null;
  private Table    tableL3            = null;
  private int      width = 20; // for now it will be square for better performance
  private boolean  initialized        = false;

  private boolean  showWire           = true;
  private boolean  drawExit           = true;

  /**Constructor*/
  public RCanvas(Table tableL1, Table tableL2, Table tableL3)
  {
    super();
    this.tableL1 = tableL1;
    this.tableL2 = tableL2;
    this.tableL3 = tableL3;
    images = new Image[10];
  }

  /**Return the size of table*/
  public Dimension getPreferredSize()
  {
    return new Dimension(tableL1.getWidth()*width, tableL1.getHeight()*width);
  }

  /**Return the square position in tables for given coordinates*/
  public Point getSquarepositionOn(int x, int y)
  {
    return new Point(x/width, y/width);
  }

  /**Main paint method*/
  public synchronized void paint(Graphics g)
  {
    if(background != null)
      g.drawImage(background, 0, 0, this);
    else
    {
      background = createImage(getSize().width, getSize().height);
    }
  }

  /**Set true when the exit must be drawn.*/
  public void setDrawExit(boolean exit)
  {
    drawExit = exit;
  }

  /**Set true when the table wires must ve drawn.*/
  public void setShowWires(boolean show)
  {
    showWire = show;
  }

  /**Init images*/
  public boolean initImages()
  {
    if(getSize().width == 0 || getSize().height == 0)
      return false;
    background = createImage(getSize().width, getSize().height);
    if(background == null)
      return false;
    backgroundGraphics = background.getGraphics();
    cFrame = createImage(getSize().width, getSize().height);
    cFrameGraphics = cFrame.getGraphics();
    thisGraphics = getGraphics();
    initialized = true;

    return true;
  }

  /**Set new layers.*/
  public void initTables(Table tableL1, Table tableL2, Table tableL3)
  {
    this.tableL1 = tableL1;
    this.tableL2 = tableL2;
    this.tableL3 = tableL3;
  }

  /**Init next frame*/
  public boolean initCurrentFrame()
  {
    Image buff = null;
    if(images[0] != null)
    {
      cFrameGraphics.drawImage(images[0], 0, 0, this);
    }

    int key = Table.BACKGROUND;

    for(int i = 0; i < tableL1.getWidth(); i++)
    {
      for(int j = 0; j < tableL1.getHeight(); j++)
      {
        if(showWire)
        {
          cFrameGraphics.setColor(Color.black);
          cFrameGraphics.fillRect(i*width, j*width, width, width);
        }

        if((key = tableL1.getIcon(i, j)) != Table.BACKGROUND)
        {
          switch(key)
          {
            case Table.BRICKS  : cFrameGraphics.setColor(Color.red); buff = images[1]; break;
            case Table.LEDDERS : cFrameGraphics.setColor(Color.cyan); buff = images[3]; break;
            case Table.EARTH   : cFrameGraphics.setColor(Color.orange); buff = images[2]; break;
            default : cFrameGraphics.setColor(Color.black); break;
          }
          if(buff != null)
            cFrameGraphics.drawImage(buff, i*width, j*width, this);
          else
            cFrameGraphics.fillRect(i*width, j*width, width, width);
          buff = null;
        }
        if((key = tableL2.getIcon(i, j)) != Table.BACKGROUND)
        {
          switch(key)
          {
            case Table.GOLD1 : cFrameGraphics.setColor(Color.yellow); buff = images[5]; break;
            case Table.HOLE1  : cFrameGraphics.setColor(Color.green); buff = images[8]; break;
            case Table.EXIT1 : if(drawExit) {cFrameGraphics.setColor(Color.blue); buff = images[4]; break;}
            default : cFrameGraphics.setColor(Color.black); break;
          }
          if(buff != null)
            cFrameGraphics.drawImage(buff, i*width, j*width, this);
          else if(showWire)
            cFrameGraphics.fillRect(i*width, j*width, width, width);
          buff = null;
        }
        if((key = tableL3.getIcon(i, j)) != Table.BACKGROUND)
        {
          switch(key)
          {
            case Table.HERO1  : cFrameGraphics.setColor(Color.pink); buff = images[6]; break;
            case Table.ENEMY1 : cFrameGraphics.setColor(Color.lightGray); buff = images[7]; break;
          }
          if(buff != null)
            cFrameGraphics.drawImage(buff, i*width, j*width, this);
          else
            cFrameGraphics.fillRect(i*width, j*width, width, width);
          buff = null;
        }
        if(showWire)
        {
          cFrameGraphics.setColor(Color.white);
          cFrameGraphics.drawRect(i*width, j*width, width, width);
        }
      }
    }

    return true;
  }

  /**Init the background image to be bigger enought.*/
  public void initBackgroundImage()
  {
    if(images[0] == null)
      return;
    Image buff = createImage(getSize().width, getSize().height);
    buff.getGraphics().drawImage(images[0], 0, 0, getSize().width, getSize().height, this);
    images[0] = buff;
  }

  /**Return true if the table is initialized*/
  public boolean isInitialized()
  {
    return initialized;
  }

  /**Draw current background to Canvas*/
  public synchronized void repaintBackBuffer()
  {
    if(background != null)
      thisGraphics.drawImage(background, 0, 0, this);
    else
    {
      background = createImage(getSize().width, getSize().height);
    }
  }

  /**Update frame*/
  public synchronized boolean updateFrame()
  {
    if(cFrame == null)
      return false;

    backgroundGraphics.drawImage(cFrame, 0, 0, this);

    return true;
  }

  /**Set the images.*/
  public void setImages(Image [] img)
  {
    images = img;
  }
}