package exion.game.runner.util;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import exion.game.runner.*;
import exion.game.runner.gui.*;

/**
 * Title:        RGame
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      exion
 * @author Matey Nenov
 * @version 1.0
 */

public class ERunnerLevelEditor extends JFrame
  implements ActionListener, Runnable, MouseListener, MouseMotionListener
{
  JMenuBar  menu = new JMenuBar();
  JMenu     file = new JMenu("File");
  JMenuItem _new = new JMenuItem("New");
  JMenuItem open = new JMenuItem("Open");
  JMenuItem save = new JMenuItem("Save");
  JMenuItem exit = new JMenuItem("Exit");

  JTextField filenameField = new JTextField("erlevel1.sav");

  Table     tableL1 = null;
  Table     tableL2 = null;
  Table     tableL3 = null;

  RCanvas   canvas  = null;

  Thread    thread  = null;

  JPanel    north   = new JPanel();
  JPanel    center  = new JPanel();
  JPanel    south   = new JPanel();
  JPanel    west    = new JPanel();

  JButton   hero        = new JButton("Hero");
  JButton   enemy       = new JButton("Enemy");
  JButton   bricks      = new JButton("Bricks");
  JButton   ledders     = new JButton("Ledders");
  JButton   backgrounds = new JButton("Background");
  JButton   earth       = new JButton("Earth");
  JButton   gold        = new JButton("Gold");
  JButton   _exit       = new JButton("Exit");

  int       currentPainting = Table.BACKGROUND;

//  String    defaultFilename = "erlevel1.sav";

  int       xsize = 30;
  int       ysize = 20;

  /**Create new Frame with menubar*/
  public ERunnerLevelEditor()
  {
    super("Exion Runner Level Editor");
    JPopupMenu.setDefaultLightWeightPopupEnabled(false);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setJMenuBar(menu);
    menu.add(file);
    file.add(_new);
    _new.addActionListener(this);
    file.add(open);
    open.addActionListener(this);
    file.add(save);
    save.addActionListener(this);
    file.add(new JSeparator());
    file.add(exit);
    exit.addActionListener(this);
    getContentPane().setLayout(new BorderLayout());
    north.setLayout(new BorderLayout());
    center.setLayout(new BorderLayout());
    west.setLayout(new BorderLayout());
    south.setLayout(new GridLayout(3, 3));

    south.add(hero);
    south.add(enemy);
    south.add(bricks);
    south.add(ledders);
    south.add(backgrounds);
    south.add(earth);
    south.add(gold);
    south.add(_exit);
    south.add(filenameField);

    hero.addActionListener(this);
    enemy.addActionListener(this);
    bricks.addActionListener(this);
    ledders.addActionListener(this);
    backgrounds.addActionListener(this);
    earth.addActionListener(this);
    gold.addActionListener(this);
    _exit.addActionListener(this);

    getContentPane().add("North", north);
    getContentPane().add("Center", center);
    getContentPane().add("West", west);
    getContentPane().add("South", south);
  }

  /**Handle all action commands*/
  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource().equals(_new))
    {
      newTable();
    }else if(e.getSource().equals(open))
    {
      if(loadTable(/*defaultFilename*/filenameField.getText().trim()))
        System.out.println("Map suxessfull loaded.");
      else
        System.out.println("Error loading map!");
    }else if(e.getSource().equals(save))
    {
      if(saveTable(/*defaultFilename*/filenameField.getText().trim()))
        System.out.println("Map suxessfull saved.");
      else
        System.out.println("Error saving current map!");
    }else if(e.getSource().equals(exit))
    {
      System.exit(0);
    }else if(e.getSource().equals(hero))
    {
      currentPainting = Table.HERO1;
    }else if(e.getSource().equals(enemy))
    {
      currentPainting = Table.ENEMY1;
    }else if(e.getSource().equals(bricks))
    {
      currentPainting = Table.BRICKS;
    }else if(e.getSource().equals(ledders))
    {
      currentPainting = Table.LEDDERS;
    }else if(e.getSource().equals(backgrounds))
    {
      currentPainting = Table.BACKGROUND;
    }else if(e.getSource().equals(earth))
    {
      currentPainting = Table.EARTH;
    }else if(e.getSource().equals(gold))
    {
      currentPainting = Table.GOLD1;
    }else if(e.getSource().equals(_exit))
    {
      currentPainting = Table.EXIT1;
    }
  }

  /**Load current table from file*/
  private synchronized boolean loadTable(String fileName)
  {
    try{
      ObjectInputStream ois = new ObjectInputStream(new FileInputStream(fileName));
      tableL1 = (Table)ois.readObject();
      tableL2 = (Table)ois.readObject();
      tableL3 = (Table)ois.readObject();
      ois.close();
      thread = new Thread(this);
      thread.start();

      return true;
    }catch(IOException ioe){System.err.println(ioe.getMessage());}
    catch(ClassNotFoundException cnfe){System.err.println(cnfe.getMessage());}

    return false;
  }

  /**Create and run new application*/
  public static void main(String[] args)
  {
    ERunnerLevelEditor mainFrame = new ERunnerLevelEditor();
    mainFrame.setSize(800, 600);
    mainFrame.setVisible(true);
  }

  /**Invoked when the mouse has been clicked on a component.*/
  public void mouseClicked(MouseEvent e) {}

  /**Invoked when the mouse enters a component.*/
  public void mouseEntered(MouseEvent e) {}

  /**Invoked when the mouse exits a component.*/
  public void mouseExited(MouseEvent e) {}

  /**Invoked when a mouse button has been pressed on a component.*/
  public void mousePressed(MouseEvent e)
  {
    Point p = canvas.getSquarepositionOn(e.getX(), e.getY());
    switch(currentPainting)
    {
      case Table.HERO1   : tableL1.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL2.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL3.setIcon(p.x, p.y, Table.HERO1);
                           break;
      case Table.ENEMY1  : tableL1.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL2.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL3.setIcon(p.x, p.y, Table.ENEMY1);
                           break;
      case Table.BRICKS  : tableL1.setIcon(p.x, p.y, Table.BRICKS);
                           tableL2.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL3.setIcon(p.x, p.y, Table.BACKGROUND);
                           break;
      case Table.LEDDERS : tableL1.setIcon(p.x, p.y, Table.LEDDERS);
                           tableL2.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL3.setIcon(p.x, p.y, Table.BACKGROUND);
                           break;
      case Table.EARTH   : tableL1.setIcon(p.x, p.y, Table.EARTH);
                           tableL2.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL3.setIcon(p.x, p.y, Table.BACKGROUND);
                           break;
      case Table.GOLD1   : tableL1.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL2.setIcon(p.x, p.y, Table.GOLD1);
                           tableL3.setIcon(p.x, p.y, Table.BACKGROUND);
                           break;
      case Table.EXIT1   : tableL1.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL2.setIcon(p.x, p.y, Table.EXIT1);
                           tableL3.setIcon(p.x, p.y, Table.BACKGROUND);
                           break;
      default            : tableL1.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL2.setIcon(p.x, p.y, Table.BACKGROUND);
                           tableL3.setIcon(p.x, p.y, Table.BACKGROUND);
                           break;
    }

    canvas.initCurrentFrame();
    canvas.updateFrame();
    canvas.repaintBackBuffer();
  }

  /**Invoked when a mouse button has been released on a component.*/
  public void mouseReleased(MouseEvent e) {}

  /*Invoked when a mouse button is pressed on a component and then dragged.*/
  public void mouseDragged(MouseEvent e)
  {
    mousePressed(e);
    e.consume();
  }

  /**Invoked when the mouse button has been moved on a component (with no buttons no down).*/
  public void mouseMoved(MouseEvent e) {}

  /**Create and init a new table*/
  private synchronized void newTable()
  {
    tableL1 = new Table(xsize, ysize);
    tableL2 = new Table(xsize, ysize);
    tableL3 = new Table(xsize, ysize);
    for(int i=0; i<xsize; i++)
    {
      for(int j=0; j<ysize; j++)
      {
        tableL1.setIcon(i, j, 0);
        tableL2.setIcon(i, j, 0);
        tableL3.setIcon(i, j, 0);
      }
    }
    thread = new Thread(this);
    thread.start();
  }

  /**Main thread used to create canvas object*/
  public synchronized void run()
  {
    if(tableL1 == null || tableL2 == null)
      return;
    if(canvas != null)
    {
      canvas.removeMouseListener(this);
      canvas.removeMouseMotionListener(this);
      north.remove(canvas);
    }
    System.out.println("create new canvas");
    canvas = new RCanvas(tableL1, tableL2, tableL3);
    canvas.addMouseListener(this);
    canvas.addMouseMotionListener(this);
    System.out.println("add new canvas to frame");
    north.add("Center", canvas);
    System.out.println("show the canvas");
    canvas.setVisible(true);
    System.out.println("call doLayout");
    validate();
    while(!canvas.isInitialized())
    {
      System.out.println("canvas.getSize = " + canvas.getSize());
      System.out.println("call canvas.initImages()");
      canvas.initImages();
      try{
        System.out.println("sleep for a while");
        if(!canvas.isInitialized())
          thread.sleep(1000);
      }catch(InterruptedException ie){System.err.println(ie.getMessage());}
    }
    System.out.println("init current frame");
    canvas.initCurrentFrame();
    System.out.println("update the frame");
    canvas.updateFrame();
    System.out.println("repaint backbuffer");
    canvas.repaintBackBuffer();
    System.out.println("canvas initialized...");
//    getContentPane().doLayout();
  }

  /**Save current table to file*/
  private boolean saveTable(String fileName)
  {
    if(tableL1 == null)
      return false;
    try{
      ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(fileName));
      oos.writeObject(tableL1);
      oos.writeObject(tableL2);
      oos.writeObject(tableL3);
      oos.flush();
      oos.close();

      return true;
    }catch(IOException ioe){System.err.println(ioe.getMessage());}

    return false;
  }
}